//
//  RCCalculator.m
//  Recursive Descent Parser for CalcProgram
//
//  Created by Rene Hexel on 20/10/06.
//  Copyright 2006 Rene Hexel. All rights reserved.
//

#import "RCCalculator.h"


@implementation RCCalculator

/**
 * initialiser with standard input
 */
- initWithStandardInput
{
	NSData *data = [ [NSFileHandle fileHandleWithStandardInput]
			 availableData ];	// read data from stdin
	return [self initWithString: [[[NSString alloc]
				initWithData: data
				    encoding: NSUTF8StringEncoding]
			autorelease]];
}


/**
 * initialiser with an expression string
 */
- initWithString: (NSString *) string
{
	self = [self init];
	
	if (self == nil) return nil;		// error allocating memory?
	
	scanner = [NSScanner scannerWithString: string];
	delimiter = [NSCharacterSet whitespaceCharacterSet];

	return [self nextToken];		// scan the next token
}


/**
 * evaluate the current expression
 *
 * expression => term { ( '+' | '-' ) term }
 */
- (double) expression
{
	double val = [self term];		// get the first term

	while ([token isEqualToString: @"+"] ||
	       [token isEqualToString: @"-"])
	{
		unichar op = [token characterAtIndex: 0];
		[self nextToken];		// read next token
		double num = [self term];	// get the next term
		
		if (op == '+')			// plus or minus?
			val += num;
		else
			val -= num;
	}
	
	return val;
}


/**
 * evaluate the current term
 *
 * term => factor { ( '*' | '/' ) factor }
 */
- (double) term
{
	double val = [self factor];		// get the first factor
	
	while ([token isEqualToString: @"*"] ||
	       [token isEqualToString: @"/"])
	{
		unichar op = [token characterAtIndex: 0];
		[self nextToken];		// read next token
		double num = [self factor];	// get the next factor
		
		if (op == '*')			// multiply or divide?
			val *= num;
		else
			val /= num;
	}
	
	return val;
}


/**
 * evaluate the current factor
 *
 * factor => number | '(' expression ')'
 */
- (double) factor
{
	double val = [token floatValue];	// a factor is either a number

	if ([token isEqualToString: @"("])	// or an expression in brackets
	{
		[self nextToken];
		val = [self expression];
		if (![token isEqualToString: @")"])
			[NSException raise: @"InvalidFactor"
				    format: @"got '%@', but expected ')'",
				            token];
	}

	[self nextToken];

	return val;
}


/**
 * scan the next token by reading all the characters up to the next delimiter
 */
- nextToken
{
	if ([scanner scanUpToCharactersFromSet: delimiter intoString: &token])
		return self;			// success!
	
	return token = nil;			// end of expression
}
@end
